@@ -2,9 +2,9 @@
 from typing import Any
 
 from unimatrix.ext import crypto
-from unimatrix.ext.crypto import oid
 
 from .algorithms import ALGORITHM_MAPPING
+from .decoder import JOSEPayloadDecoder
 from .header import JOSEHeader
 from .jwt import JSONWebToken
 from .signature import JOSESignature
@@ -35,7 +35,8 @@ class JSONWebSignature(crypto.Signable):
         return cls(
             header,
             header.get_payload(payload),
-            JOSESignature(base64url_decode(signature), header.algorithm)
+            JOSESignature(base64url_decode(signature), header.algorithm,
+                keyid=header.kid)
         )
 
     def __init__(self, header, payload, signature=None):
@@ -45,6 +46,19 @@ class JSONWebSignature(crypto.Signable):
         if isinstance(payload, JSONWebToken):
             self.header.setdefault('typ', 'JWT')
 
+    async def decode(self, decoder: JOSEPayloadDecoder) -> JSONWebToken:
+        """Invoke the :class:`~JOSEPayloadDecoder` interface to verify the
+        signature and return the decoded :class:`JSONWebToken`.
+
+        .. warning::
+
+            The claims made by the :class:`JSONWebToken` are **not** verified.
+            It is up to the caller to decide when and in what order to do
+            the verification.
+        """
+        await decoder.verify(self.header, self)
+        return self.payload
+
     def get_jose_header(self, signer: crypto.Signer):
         """Return a :class:`~unimatrix.ext.crypto.lib.jose.JOSEHeader` instance
         based on the metadata of :class:`~unimatrix.ext.crypto.Signer`
@@ -59,6 +73,21 @@ class JSONWebSignature(crypto.Signable):
         """
         return self.get_jose_header(signer) + b'.' + bytes(self.payload)
 
+    def is_signed(self) -> bool:
+        """Return a boolean indicating if the :class:`JSONWebSignature`
+        is signed.
+        """
+        return self.signature is not None
+
+    async def sign(self, *args, **kwargs):
+        """Sign the :class:`JSONWebSignature` and ensure that the `kid`
+        header parameter is set, if present on the signing key.
+        """
+        sig = await super().sign(*args, **kwargs)
+        if sig.keyid:
+            self.header['kid'] = sig.keyid 
+        return sig
+
     async def verify(self, key):
         """Verify the :class:`JSONWebSignature` with the given key. Return
         a boolean indicating if the signature was valid.
