@@ -15,8 +15,8 @@ class PayloadDecodingTestCase(unittest.TestCase):
     def setUp(self):
         self.signer = HMACSigner()
         self.jws = jose.jwt.sync({'sub': 'Foo Bar'}, signer=self.signer)
-        ioc.provide('JOSEPayloadDecoder', JOSEPayloadDecoder())
-        ioc.provide('JOSEKeyResolver', JOSEKeyResolver())
+        ioc.provide('JOSEPayloadDecoder', JOSEPayloadDecoder(), force=True)
+        ioc.provide('JOSEKeyResolver', JOSEKeyResolver(), force=True)
         ioc.provide('CryptoKeyStore', AsyncMock())
 
     def test_decode_jws_payload(self):
